import lib-sampler.glsl
import lib-env.glsl
import lib-normal.glsl
import lib-alpha.glsl
import lib-pbr.glsl

//: state cull_face off

//-------------------------- Diffuse Map --------------------------//

//: param auto channel_basecolor
uniform sampler2D diffuseMapSampler;

//: param auto channel_opacity
uniform sampler2D opacitySampler;
//-------- Normal Map ---------------------------------------------------//

//: param auto channel_user0
uniform sampler2D envmapmaskSampler;

//: param auto channel_specular
uniform sampler2D specularSampler;

//: param auto channel_glossiness
uniform sampler2D glossinessSampler;

//-------- Envmap Parameters ---------------------------------------------------//

//: param custom { "default": 1, "label": "Envmap Color", "widget": "color" }
uniform vec3 fEnvmapColor;

//: param custom { "default": true, "label": "Use Specular" }
uniform bool bUseSpecular;

//: param custom { "default": true, "label": "Enable Envmap" }
uniform bool bUseEnvmap;

//-------- Specular Parameters ---------------------------------------------------//

//: param custom { "default": 1, "label": "Specular Color", "widget": "color" }
uniform vec3 fSpecularColor;

//: param custom { "default": 16.0, "label": "Specular Exponent", "min": 0.0, "max": 512.0 }
uniform float fSpecularExponent;

//: param custom { "default": 1.0, "label": "Specular Boost", "min": 0.0, "max": 16.0 }
uniform float fSpecularScale;

//-------- Ambient ---------------------------------------------------//

//: param custom { "default": 0, "label": "Ambient Color", "widget": "color" }
uniform vec3 fAmbientColor;

//: param custom { "default": 1.0, "label": "Ambient Scale", "min": 1.0, "max": 10.0 }
uniform float fAmbientScale;

//: param custom { "default": true, "label": "IBL Lighting" }
uniform bool bUseIBL;


//-------- Lights ----------------------------------------------------//

//: param custom { "default": [10.0, 10.0, 10.0], "label": "Light Position", "min": -20, "max": 20 }
uniform vec3 lightPosition_1;

//: param custom { "default": 1, "label": "Light Color", "widget": "color" }
uniform vec3 lightColor_1;


//------------------------------------------------------------------//
//-------------------------- Data Stucts ---------------------------//
//------------------------------------------------------------------//


struct lightProp
{
	vec3 lightVector;
	vec3 lightColor;
};

lightProp lightArray[1];

// Lights! Method based on Xoluil Shader ( http://www.laurenscorijn.com/xoliulshader ) Thanks mate :)
void PropagateLights( vec3 position )
{
	lightArray[0].lightVector = lightPosition_1 - position;
	lightArray[0].lightColor = lightColor_1;
}

void DoLighting( vec3 N, vec3 L, vec3 V, vec3 LightColor, float flSpecularExponent, inout vec3 finalDiffuse, inout vec3 finalSpecular)
{
	// Normalize
	L = normalize(L);

	float NdotL = dot(N, L);
	float halfLambert = NdotL * 0.5 + 0.5;

	vec3 diffuseLight = vec3(0.0);

	diffuseLight += halfLambert;

	diffuseLight *= LightColor; // Modulate with light color

	finalDiffuse += diffuseLight; // Output our diffuse lighting

	NdotL = clamp(NdotL,0.0,1.0);

	vec3 R = reflect( V, N ); // No half-vector so this is consistent in look with ps2.0
	float RdotL = clamp(dot( L, -R ),0.0,1.0);
	vec3 SpecularLighting = vec3(0.0);

	//if( bDoSpecularWarp == true)
	//{
	//	SpecularLighting = texture2D( specularWarpSampler, vec2( RdotL, flSpecularExponent ) ).rgb;
	//}
	//else
	//{
		flSpecularExponent *= fSpecularExponent; // fSpecularExponent is from the UI Spinner
		float flSpecularIntensity = NdotL * pow( RdotL, flSpecularExponent );
		SpecularLighting = vec3( flSpecularIntensity, flSpecularIntensity, flSpecularIntensity );
	//}

	SpecularLighting *= LightColor;
	finalSpecular += SpecularLighting;

}

vec4 shade(V2F inputs)
{

//All tangent space normals

  vec3 normalTGT = getTSNormal(inputs.tex_coord);

//Compute a world space normal from the tangent space one

  	vec3 normalWS = normalize(normalTGT.x * inputs.tangent + normalTGT.y * inputs.bitangent + normalTGT.z * inputs.normal);
	if (isBackFace()) {
    	normalWS = -normalWS;
  	}

	PropagateLights(inputs.position.xyz); // Bring in lights

	vec3 eye_vec = normalize(camera_pos - inputs.position);

	vec3 normalRF = eye_vec - 2 * dot(eye_vec,normalWS) * normalWS;

	vec2 texCoord = inputs.tex_coord;
	vec3 V = eye_vec;

	float flEnvmapMask  = getRoughness(envmapmaskSampler,texCoord);;

	if (bUseSpecular) {
			flEnvmapMask    = getRoughness(specularSampler,texCoord);
	}
	float flSpecularMask      = getRoughness(specularSampler,texCoord);
	float flSpecularExponent  = getRoughness(glossinessSampler,texCoord);


	float VdotN = clamp( dot( V, normalWS ),0.0,1.0);
	//if (bUseFresnelRangesMap)
	//{
	//	FresnelTerm = texture2D( fresnelwarp, vec2( VdotN, 0.5 ) ).rgb;
	//}

	vec4 albedo = texture2D(diffuseMapSampler, texCoord);
	albedo.rgb += DEFAULT_BASE_COLOR * (1.0 - albedo.a);

	vec3 diffuseColor = vec3(0.5); // global diffuse color just so we can see something by default ( gets overwritten by albedo texturemap if enabled )
	diffuseColor = albedo.rgb; // assign the diffuse texture to our global diffuse color



	//if (bDoFresnelWarp)
	//{
	//	vec3 warpDiffuseTex = diffuseColor;
	//	vec3 colorWarp = tex3D( fresnelColorWarpSampler, warpDiffuseTex ).xyz;
	//	float flFresnelColorWarp = flDiffuseWarpMask * FresnelTerm.g;
	//	diffuseColor = mix( diffuseColor, colorWarp, flFresnelColorWarp );
	//}

	vec3 ambient = fAmbientColor + fAmbientScale;
	if(bUseIBL)
	{
		ambient = envIrradiance(normalWS) * fAmbientScale;
	}


	vec3 finalDiffuse = vec3(0.0);
	vec3 finalSpecular = vec3(0.0);
	vec3 finalEnvmap = vec3 (0.0);

	//finalDiffuse += ambient;
	vec3 final;

	DoLighting( normalWS, lightArray[0].lightVector, V, lightArray[0].lightColor, flSpecularExponent, finalDiffuse, finalSpecular);

	final = (finalDiffuse * diffuseColor * ambient);

	finalEnvmap = envSampleLOD(-normalRF, 2);

	vec3 cEnvmap = finalEnvmap * fEnvmapColor;
	cEnvmap *= flEnvmapMask;

	vec3 cSpecular = finalSpecular * fSpecularScale;
	cSpecular *= flSpecularMask;
	cSpecular *= fSpecularColor;

	final += cSpecular;

	if (bUseEnvmap) {
		final += cEnvmap;
	}

	vec3 EnvReflection = vec3(0.0);

	final *= getShadowFactor();

	return vec4(final,1);

}

void shadeShadow(V2F inputs)
{
}
